/*
 * dappprof - profile user and library function usage.
 *            Written using DTrace (Solaris 10 3/05).
 *
 * The default output traces user functions as they are called. Options
 *  can be used to examine libraries and timestamps.
 *
 * $Id: dappprof 65 2007-10-04 11:09:40Z brendan $
 *
 * The elapsed times are interesting, to help identify calls that take
 *  some time to complete (during which the process may have context
 *  switched off the CPU). 
 *
 * SEE ALSO: dapptrace      # DTraceToolkit
 *           dtruss         # DTraceToolkit
 *           apptrace           
 *
 * COPYRIGHT: Copyright (c) 2005 Brendan Gregg.
 *
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 *
 * Author: Brendan Gregg  [Sydney, Australia]
 *
 * 16-May-2005   Brendan Gregg   Created this.
 * 17-Jul-2005	   "      "	Last update.
 * 29-Oct-2008   Tom Erickson    (adapted to Chime)
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 * Save syscall entry info
 */
pid$target:??MODULE??::entry
{
        /* set function depth */
        this->fdepth = ++fdepth[probefunc];

        /* set start details */
        self->start[probefunc,this->fdepth] = timestamp;
        self->vstart[probefunc,this->fdepth] = vtimestamp;

        /* count occurances */
        @Counts[??T??] = count();
}

pid$target:??MODULE??::return
/self->start[probefunc,fdepth[probefunc]]/
{
        /* fetch function depth */
        this->fdepth = fdepth[probefunc];

        /* calculate elapsed time */
        this->elapsed = timestamp - self->start[probefunc,this->fdepth];
        self->start[probefunc,this->fdepth] = 0;
        this->cpu = vtimestamp - self->vstart[probefunc,this->fdepth];
        self->vstart[probefunc,this->fdepth] = 0;

        /* save elapsed times */
        @Elapsed[??T??] = sum(this->elapsed);

        /* save cpu times */
        @CPU[??T??] = sum(this->cpu);
}

??MODULE a.out
